{
    package DBD::Oracle;

    require DBI;

    require DynaLoader;
    @ISA = qw(DynaLoader);
    @EXPORT = qw(); # Do NOT @EXPORT anything.

#   $Id: Oracle.pm,v 1.5 1994/10/20 16:42:55 timbo Exp $
#
#   Copyright (c) 1994, Tim Bunce
#
#   You may distribute under the terms of either the GNU General Public
#   License or the Artistic License, as specified in the Perl README file.

    bootstrap DBD::Oracle;

    unless ($ENV{'ORACLE_HOME'}){
	foreach(qw(/usr/oracle /opt/oracle /usr/soft/oracle)){
	    $ENV{'ORACLE_HOME'}=$_,last if -d "$_/rdbms/lib";
	}
	die "Unable to determine ORACLE_HOME" unless $ENV{'ORACLE_HOME'};
	warn "ORACLE_HOME set to $ENV{'ORACLE_HOME'}\n";
    }

    $drh = undef;	# holds driver handle once initialised

    sub driver{
	return $drh if $drh;
	my($class, $attr) = @_;
	$class .= "::dr";
	($drh) = DBI::_new_drh($class, {
	    'Name' => 'Oracle',
	    'Version' => '$Revision: 1.5 $',
	    'Attribution' => 'Oracle DBD by Tim Bunce',
	    });
	$drh;
    }

    1;
}


{   package DBD::Oracle::dr; # ====== DRIVER ======
    use strict;

    sub connect { # normally overridden, but a handy default
	my($drh, $dbname, $user, $auth)= @_;

	if ($dbname){ # else keep current env settings
	    if ($dbname !~ /:/){
		@ENV{qw(ORACLE_SID TWO_TASK)} = ($dbname, '');
	    } else {
		@ENV{qw(ORACLE_SID TWO_TASK)} = ('', $dbname);
	    }
	}

	# Call Oracle OCI orlon func in Oracle.xs file.
	# Store returned structure pointer as private data
	# in the $dbh created by DBI::_new_dbh() below.

	my($imp_dbh) = orlon($drh, $user, $auth);
	return undef unless $imp_dbh;

	my($this) = DBI::_new_dbh($drh, {
	    'Name' => $dbname,
	    'User' => $user,
	    }, $imp_dbh);
	$this;
    }

    sub errstr { &DBD::Oracle::errstr(@_) }
}


{   package DBD::Oracle::db; # ====== DATABASE ======
    use strict;

    sub prepare {
	my($dbh, $statement)= @_;

	# Call Oracle OCI oparse func in Oracle.xs file.
	# (This will actually also call oopen for you.)
	# Store returned structure pointer as private data
	# in the $sth created by DBI::_new_sth() below.

	my($imp_sth) = oparse($dbh, $statement);
	return undef unless $imp_sth;

	my($outer, $sth) = DBI::_new_sth($dbh, {
	    'Statement' => $statement,
	    }, $imp_sth);
	$outer;
    }

    sub errstr { &DBD::Oracle::errstr(@_) }
}


{   package DBD::Oracle::st; # ====== STATEMENT ======
    use strict; no strict 'refs'; # cause problems with filehandles

    sub execute {
	my($sth, $dir) = @_;
	$sth->finish;
	warn "execute not implemented yet\n";
	1;
    }

    sub fetchrow {
	my($sth) = @_;
	warn "fetchrow not implemented yet\n";
	return ();
    }

    sub finish {
	my($sth) = @_;
	undef;
    }

    sub _FETCH {
	my ($sth, $attrib) = @_;
	# In reality this would interrogate the database engine to
	# either return dynamic values that cannot be precomputed
	# or fetch and cache attribute values too expensive to prefetch.
	warn "fetch unknown attribute '$attrib' of $sth\n";
	return undef; # unknown attribute
    }

    sub STORE {
	my ($sth, $attrib, $value) = @_;
	# would normally validate and only store known attributes
	warn "store unknown attribute '$attrib' of $sth\n";
	$sth->{$attrib} = $value;
    }

    sub errstr { &DBD::Oracle::errstr(@_) }
}

1;
